----------------------------------------------------------------------------------
-- Company: Univ Lyon, UJM-Saint-Etienne, CNRS, Laboratoire Hubert Curien UMR 5516, F-42023, SAINT-ETIENNE, France
-- Creator: Cédric Marchand 
-- 
-- Create Date:    08:42:23 11/13/2015 
-- Design Name: 
-- Module Name:    mixrow_d - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity mixrow_d is
  port(
    state_i : in std_logic_vector(63 downto 0);
    state_o : out std_logic_vector(63 downto 0)
  );
end mixrow_d;

architecture Behavioral of mixrow_d is

  COMPONENT x2
	PORT(
		nible_i : IN std_logic_vector(3 downto 0);          
		nible_o : OUT std_logic_vector(3 downto 0)
		);
	END COMPONENT;
  
  COMPONENT x4
	PORT(
		nible_i : IN std_logic_vector(3 downto 0);          
		nible_o : OUT std_logic_vector(3 downto 0)
		);
	END COMPONENT;
  
  signal x411_i : std_logic_vector(3 downto 0);
  signal x421_i : std_logic_vector(3 downto 0);
  signal x431_i : std_logic_vector(3 downto 0);
  signal x441_i : std_logic_vector(3 downto 0);
  signal x411_o : std_logic_vector(3 downto 0);
  signal x421_o : std_logic_vector(3 downto 0);
  signal x431_o : std_logic_vector(3 downto 0);
  signal x441_o : std_logic_vector(3 downto 0);
  signal x412_i : std_logic_vector(3 downto 0);
  signal x422_i : std_logic_vector(3 downto 0);
  signal x432_i : std_logic_vector(3 downto 0);
  signal x442_i : std_logic_vector(3 downto 0);
  signal x412_o : std_logic_vector(3 downto 0);
  signal x422_o : std_logic_vector(3 downto 0);
  signal x432_o : std_logic_vector(3 downto 0);
  signal x442_o : std_logic_vector(3 downto 0);
  signal x211_o : std_logic_vector(3 downto 0);
  signal x221_o : std_logic_vector(3 downto 0);
  signal x231_o : std_logic_vector(3 downto 0);
  signal x241_o : std_logic_vector(3 downto 0);
  signal x212_o : std_logic_vector(3 downto 0);
  signal x222_o : std_logic_vector(3 downto 0);
  signal x232_o : std_logic_vector(3 downto 0);
  signal x242_o : std_logic_vector(3 downto 0);

begin

  state_o(63 downto 16) <= state_i(47 downto 0);
  -- state(3 downto 0)
  x411_i <= state_i(3 downto 0) xor state_i(51 downto 48);
  Inst_x411 : x4 PORT MAP(
		nible_i => x411_i,
		nible_o => x411_o
	);
  
  Inst_x211: x2 PORT MAP(
		nible_i => x411_o,
		nible_o => x211_o
	);
  
  x412_i <= state_i(19 downto 16) xor state_i(35 downto 32);
  Inst_x412 : x4 PORT MAP(
		nible_i => x412_i,
		nible_o => x412_o
	);
  
  Inst_x212: x2 PORT MAP(
		nible_i => x412_o,
		nible_o => x212_o
	);
  
  state_o(3 downto 0) <= x211_o xor x411_o xor x411_i xor x212_o xor x412_i;
  
  --state(7 downto 4)
  x421_i <= state_i(7 downto 4) xor state_i(55 downto 52);
  Inst_x421 : x4 PORT MAP(
		nible_i => x421_i,
		nible_o => x421_o
	);
  
  Inst_x221: x2 PORT MAP(
		nible_i => x421_o,
		nible_o => x221_o
	);
  
  x422_i <= state_i(23 downto 20) xor state_i(39 downto 36);
  Inst_x422 : x4 PORT MAP(
		nible_i => x422_i,
		nible_o => x422_o
	);
  
  Inst_x222: x2 PORT MAP(
		nible_i => x422_o,
		nible_o => x222_o
	);
  
  state_o(7 downto 4) <= x221_o xor x421_o xor x421_i xor x222_o xor x422_i;
  
  --state(11 downto 8)
  x431_i <= state_i(11 downto 8) xor state_i(59 downto 56);
  Inst_x431 : x4 PORT MAP(
		nible_i => x431_i,
		nible_o => x431_o
	);
  
  Inst_x231: x2 PORT MAP(
		nible_i => x431_o,
		nible_o => x231_o
	);
  
  x432_i <= state_i(27 downto 24) xor state_i(43 downto 40);
  Inst_x432 : x4 PORT MAP(
		nible_i => x432_i,
		nible_o => x432_o
	);
  
  Inst_x232: x2 PORT MAP(
		nible_i => x432_o,
		nible_o => x232_o
	);
  
  state_o(11 downto 8) <= x231_o xor x431_o xor x431_i xor x232_o xor x432_i;
  
  --state(15 downto 12)
  x441_i <= state_i(15 downto 12) xor state_i(63 downto 60);
  Inst_x441 : x4 PORT MAP(
		nible_i => x441_i,
		nible_o => x441_o
	);
  
  Inst_x241: x2 PORT MAP(
		nible_i => x441_o,
		nible_o => x241_o
	);
  
  x442_i <= state_i(31 downto 28) xor state_i(47 downto 44);
  Inst_x442 : x4 PORT MAP(
		nible_i => x442_i,
		nible_o => x442_o
	);
  
  Inst_x242: x2 PORT MAP(
		nible_i => x442_o,
		nible_o => x242_o
	);
  
  state_o(15 downto 12) <= x241_o xor x441_o xor x441_i xor x242_o xor x442_i;

end Behavioral;

